import { getContext, setContext } from 'svelte';
import { LocalizationService } from './service.svelte.js';
class Factory {
    static __instance = undefined;
    static __config = undefined;
    static __commonServiceConfig = undefined;
    static configure(config) {
        Factory.__config = config;
    }
    static get config() {
        if (!Factory.__config) {
            throw new Error('Localization Service Factory not initialized, use configure() first');
        }
        return Factory.__config;
    }
    static setCommonServiceConfig(config) {
        if (config.availableLocales.length === 0) {
            throw new Error('At least one available locale must be provided');
        }
        Factory.__commonServiceConfig = config;
    }
    static get commonServiceConfig() {
        if (!Factory.__commonServiceConfig) {
            throw new Error('Localization Common Service Config not initialized, use setCommonServiceConfig() first');
        }
        return Factory.__commonServiceConfig;
    }
    static createService(config) {
        if (!this.config.browser) {
            return new LocalizationService(config);
        }
        if (!this.__instance) {
            this.__instance = new LocalizationService(config);
        }
        return this.__instance;
    }
    static getContextService() {
        if (!this.config.browser) {
            const service = getContext(this.config.contextName)?.service();
            if (!service) {
                throw new Error(`Localization service not found in context ${this.config.contextName}`);
            }
            return service;
        }
        if (!this.__instance) {
            throw new Error('Localization service is not initialized, use initialLoadLocalizations() first');
        }
        return this.__instance;
    }
    static setContextService(service) {
        setContext(this.config.contextName, { service: () => service });
    }
}
export function importLoadFactory(importDirPath, importLoads) {
    return (importFilePath) => {
        return async (locale) => {
            const importLocalizationPath = `${importDirPath}/${locale}/${importFilePath}`;
            const importFunc = importLoads[importLocalizationPath];
            if (!importFunc)
                return undefined;
            const data = (await importFunc());
            return data.default;
        };
    };
}
export const LocalizationFactory = {
    configure(config) {
        Factory.configure(config);
    },
    get config() {
        return Factory.config;
    },
    setCommonServiceConfig(config) {
        Factory.setCommonServiceConfig(config);
    },
    get commonServiceConfig() {
        return Factory.commonServiceConfig;
    },
    importLoaderFactory() {
        return importLoadFactory(Factory.config.importDirPath, Factory.config.importLoads);
    },
    setContextService(service) {
        Factory.setContextService(service);
    },
    getContextService() {
        return Factory.getContextService();
    },
    async initialLoadLocalizations(config, pathname) {
        const _config = { ...Factory.commonServiceConfig, ...config };
        if (!_config.activeLocale) {
            throw new Error('activeLocale is required in initialLoadLocalizations');
        }
        const service = Factory.createService(_config);
        await service.loadLocalizations(pathname);
        return service;
    }
};
