import { IntlMessageFormat } from 'intl-messageformat';
import { SvelteMap } from 'svelte/reactivity';
import { namedFormat } from './string.js';
export class LocalizationService {
    #config;
    #availableLocales;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    #activeLocale = $state(undefined);
    // Loaded localizations: locale -> key -> value, flattened and expanded
    #localizations = new SvelteMap();
    // Which localizations have alread been loaded: locale -> loader -> boolean
    #executedLoaders = new Map();
    #logger;
    #lastLoadedPathname = undefined;
    constructor(config) {
        this.#config = config;
        if (config.availableLocales.length === 0) {
            throw new Error('At least one available locale must be provided');
        }
        this.#availableLocales = config.availableLocales;
        this.#activeLocale = config.activeLocale;
        this.#logger = config.logger;
    }
    get availableLocales() {
        return this.#availableLocales;
    }
    getActiveLocale = () => {
        return this.#activeLocale;
    };
    setActiveLocale = (locale) => {
        if (this.#activeLocale === locale)
            return;
        if (!this.#availableLocales.includes(locale)) {
            throw new Error(`Trying to set locale ${locale} which is not available`);
        }
        this.#activeLocale = locale;
        if (this.#lastLoadedPathname) {
            this.#logger?.debug(`Reloading localizations for locale ${locale} and pathname ${this.#lastLoadedPathname}`);
            this.loadLocalizations(this.#lastLoadedPathname);
        }
    };
    text = (key, params) => {
        if (!this.#activeLocale)
            return key;
        const formatText = this.#localizations.get(this.#activeLocale)?.get(key);
        if (!formatText) {
            this.#logger?.debug(`Translation not found for key for the currently active locale ${this.#activeLocale}: ${key}`);
            const value = params?.default ?? key;
            const prepare = this.#config.prepare ?? prepareNamedFormat;
            return prepare(this.#activeLocale, value)(params);
        }
        return formatText(params);
    };
    getPSText = (prefix, suffix) => {
        const text = this.text;
        return (key, params) => {
            const prefixKey = prefix ? `${prefix}.${key}` : key;
            const suffixKey = suffix ? `${prefixKey}.${suffix}` : prefixKey;
            return text(suffixKey, params);
        };
    };
    loadLocalizations = async (pathname) => {
        if (pathname === '')
            throw new Error('Pathname must not be empty');
        // Get relevant loaders
        const loaders = this.getRelevantLoaders(pathname);
        const loadLocale = this.#activeLocale;
        for (const loader of loaders) {
            if (this.#executedLoaders.get(loadLocale)?.has(loader))
                continue;
            let localeExecutedLoaders = this.#executedLoaders.get(loadLocale);
            if (!localeExecutedLoaders) {
                localeExecutedLoaders = new Set([loader]);
                this.#executedLoaders.set(loadLocale, localeExecutedLoaders);
            }
            else {
                localeExecutedLoaders.add(loader);
            }
            try {
                const data = await loader.load(loadLocale);
                if (!data) {
                    this.#logger?.debug(`No data loaded for locale ${loadLocale} and loader ${loader.key} [${loader.routes ?? '/*'}]`);
                    continue;
                }
                const flatStringData = this.flattenLocalizationWithPrefix(loader.key, data);
                let localeLocalization = this.#localizations.get(loadLocale);
                if (!localeLocalization) {
                    localeLocalization = new SvelteMap();
                    this.#localizations.set(loadLocale, localeLocalization);
                }
                const prepare = this.#config.prepare ?? prepareNamedFormat;
                for (const [key, value] of flatStringData) {
                    localeLocalization.set(key, prepare(loadLocale, value));
                }
                // Successfully loaded, no need to try other locales
                break;
            }
            catch (error) {
                this.#logger?.error(`Failed to load localization for locale ${loadLocale} and loader ${loader.key}/${loader.routes ?? 'all'}: ${error}`);
            }
        }
        this.#lastLoadedPathname = pathname;
    };
    getRelevantLoaders(pathname) {
        return (this.#config.loaders?.filter((loader) => {
            if (!loader.routes || loader.routes.length === 0)
                return true;
            return loader.routes?.some((route) => {
                if (typeof route === 'string') {
                    return pathname.startsWith(route);
                }
                return route.test(pathname);
            });
        }) || []);
    }
    flattenLocalizationWithPrefix(prefixKey, localization) {
        const flatLocalization = this.flattenLocalization(localization);
        const prefix = prefixKey || prefixKey == '' ? `${prefixKey}.` : '';
        const result = new Map();
        for (const [key, value] of flatLocalization) {
            result.set(`${prefix}${key}`, value);
        }
        return result;
    }
    flattenLocalization(localization) {
        const result = new Map();
        for (const key in localization) {
            const value = localization[key];
            if (typeof value === 'object') {
                const nested = this.flattenLocalization(value);
                for (const [nestedKey, nestedValue] of nested) {
                    result.set(key + '.' + nestedKey, nestedValue);
                }
            }
            else {
                result.set(key, value);
            }
        }
        return result;
    }
}
export const prepareNamedFormat = (_, value) => {
    return function (params) {
        return namedFormat(value, params);
    };
};
export const prepareICUFormat = (locale, value) => {
    const msg = new IntlMessageFormat(value, locale);
    return function (params) {
        return msg.format(params);
    };
};
